/*******************************************************************************
 * I9 TECHNOLOGIES CONFIDENTIAL
 * __________________
 * Copyright © 2013-2016 i9 Technologies 
 * All Rights Reserved.
 * 
 * NOTICE:  All information contained herein is, and remains the property 
 * of i9 Technologies and its suppliers, if any.  The intellectual and technical 
 * concepts contained herein are proprietary to i9 Technologies and its 
 * suppliers and may be covered by U.S. and Foreign Patents, patents in 
 * process, and are protected by trade secret or copyright law. Dissemination 
 * of this information or reproduction of this material is strictly forbidden 
 * unless prior written permission is obtained from i9 Technologies.
 ******************************************************************************/
package com.xmlnamespace.panel.vaadin.shared.ui;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents a menu item in the DropDownMenu
 */
public class DropDownMenuItem implements Serializable {

    private static final long serialVersionUID = 1L;

    /** The id. */
    private double id;

    /** The styles. */
    private List<String> styles = new ArrayList<String>();

    /** The change listeners. */
    private List<DropDownMenuItemChangeListener> changeListeners = new ArrayList<DropDownMenuItemChangeListener>();

    /** The name. */
    private String name = "";

    /** The type. */
    private String type = "";

    /**
     * Instantiates a new drop down menu item.
     */
    public DropDownMenuItem() {
        id = Math.random();
    }

    /**
     * Instantiates a new menu item.
     *
     * @param name
     *            the name
     * @param type
     *            the type
     */
    public DropDownMenuItem(String name, String type) {
        this();
        this.name = name;
        this.type = type;
    }

    /**
     * Gets the name.
     *
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * Sets the name.
     *
     * @param name
     *            the new name
     */
    public void setName(String name) {
        this.name = name;
        fireChangeListeners();
    }

    /**
     * Gets the type.
     *
     * @return the type
     */
    public String getType() {
        return type;
    }

    /**
     * Sets the type.
     *
     * @param type
     *            the new type
     */
    public void setType(String type) {
        this.type = type;
        fireChangeListeners();
    }

    /**
     * Gets the id.
     *
     * @return the id
     */
    public double getId() {
        return id;
    }

    /**
     * Sets the id.
     *
     * @param id
     *            the new id
     */
    public void setId(double id) {
        this.id = id;
    }

    /**
     * Adds a style.
     *
     * @param style
     *            the style
     */
    public void addStyle(String style) {
        if (!styles.contains(style)) {
            styles.add(style);
            fireChangeListeners();
        }
    }

    /**
     * Removes a style.
     *
     * @param style
     *            the style
     */
    public void removeStyle(String style) {
        styles.remove(style);
        fireChangeListeners();
    }

    /**
     * Sets the styles.
     * 
     * For internal use only. Will not update the item styles when called.
     *
     * @param styles
     *            the styles
     * @return the list
     */
    public void setStyles(List<String> styles) {
        this.styles = styles;
    }

    /**
     * Gets the styles.
     *
     * @return the styles
     */
    public List<String> getStyles() {
        return styles;
    }

    /**
     * Adds a change listener.
     *
     * @param listener
     *            the listener
     */
    public void addChangeListener(DropDownMenuItemChangeListener listener) {
        changeListeners.add(listener);
    }

    /**
     * Remove a change listener.
     *
     * @param listener
     *            the listener
     */
    public void removeChangeListener(DropDownMenuItemChangeListener listener) {
        changeListeners.remove(listener);
    }

    /**
     * Gets the change listeners.
     *
     * @return the change listeners
     */
    public List<DropDownMenuItemChangeListener> getChangeListeners() {
        return changeListeners;
    }

    /**
     * Fire change listeners.
     */
    protected void fireChangeListeners() {
        for (DropDownMenuItemChangeListener listener : changeListeners) {
            listener.menuItemChanged(new DropDownMenuItemChangedEvent(this));
        }
    }

    /**
     * The listener interface for receiving DropDownMenuItemItemChange events.
     * The class that is interested in processing a DropDownMenuItemItemChange
     * event implements this interface, and the object created with that class
     * is registered with a component using the component's
     * <code>addDropDownMenuItemItemChangeListener<code> method. When
     * the DropDownMenuItemItemChange event occurs, that object's appropriate
     * method is invoked.
     *
     * @see MenuItemItemChangeEvent
     */
    public static interface DropDownMenuItemChangeListener {

        /**
         * Called when a menu item changes.
         *
         * @param event
         *            the menu item changed event
         */
        public void menuItemChanged(DropDownMenuItemChangedEvent event);

    }

    /**
     * Event fired when the menu item changes.
     * 
     * This is used to inform the DropDownMenuItem that one of its items has
     * changed and an event needs to be sent to the client or the state needs to
     * be marked as dirty.
     */
    public static class DropDownMenuItemChangedEvent {

        /** The item. */
        private DropDownMenuItem item;

        /**
         * Instantiates a new menu item changed event.
         *
         * @param item
         *            the item that has changed
         */
        public DropDownMenuItemChangedEvent(DropDownMenuItem item) {
            this.item = item;
        }

        /**
         * Gets the item that has changed.
         *
         * @return the item that has changed
         */
        public DropDownMenuItem getItem() {
            return item;
        }

        /**
         * Sets the item that has changed.
         *
         * @param item
         *            the item that has changed
         */
        public void setItem(DropDownMenuItem item) {
            this.item = item;
        }

    }

}