/*******************************************************************************
 * I9 TECHNOLOGIES CONFIDENTIAL
 * __________________
 * Copyright © 2013 i9 Technologies 
 * All Rights Reserved.
 * 
 * NOTICE:  All information contained herein is, and remains the property 
 * of i9 Technologies and its suppliers, if any.  The intellectual and technical 
 * concepts contained herein are proprietary to i9 Technologies and its 
 * suppliers and may be covered by U.S. and Foreign Patents, patents in 
 * process, and are protected by trade secret or copyright law. Dissemination 
 * of this information or reproduction of this material is strictly forbidden 
 * unless prior written permission is obtained from i9 Technologies.
 ******************************************************************************/
package com.xmlnamespace.panel.vaadin.ui;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.vaadin.event.MouseEvents.ClickEvent;
import com.vaadin.event.MouseEvents.ClickListener;
import com.vaadin.event.Transferable;
import com.vaadin.event.TransferableImpl;
import com.vaadin.event.dd.DragSource;
import com.vaadin.server.Resource;
import com.vaadin.server.ResourceReference;
import com.vaadin.shared.MouseEventDetails;
import com.vaadin.shared.communication.URLReference;
import com.vaadin.ui.AbstractComponent;
import com.vaadin.ui.Component;
import com.xmlnamespace.panel.vaadin.shared.ui.CallButtonItem;
import com.xmlnamespace.panel.vaadin.shared.ui.CallComponentClientRpc;
import com.xmlnamespace.panel.vaadin.shared.ui.CallComponentItem;
import com.xmlnamespace.panel.vaadin.shared.ui.CallComponentItem.CallComponentItemChangeListener;
import com.xmlnamespace.panel.vaadin.shared.ui.CallComponentItem.CallComponentItemChangedEvent;
import com.xmlnamespace.panel.vaadin.shared.ui.CallComponentServerRpc;
import com.xmlnamespace.panel.vaadin.shared.ui.CallComponentState;

/**
 * Component used to display an iSymphony call
 * Provides listeners to detect click events on call buttons
 * Provides drag functionality that allows users to specify the object that is dragged.
 */
public class CallComponent extends AbstractComponent implements
        CallComponentItemChangeListener, DragSource {
        
    /** The call button item map. */
    private Map<Double, CallButtonItem> callButtonItemMap = new HashMap<Double, CallButtonItem>();
        
    /** The call drag object. */
    private Object callDragObject;
        
    /** The timer start time. */
    private long startTime = -1;
    
    /** The rpc. */
    private CallComponentServerRpc rpc = new CallComponentServerRpc() {

        /** The Constant serialVersionUID. */
        private static final long serialVersionUID = 1L;

        /* (non-Javadoc)
         * @see com.xmlnamespace.panel.vaadin.shared.ui.CallComponentServerRpc#callButtonItemClicked(double, com.vaadin.shared.MouseEventDetails)
         */
        @Override
        public void callButtonItemClicked(double itemId,
                MouseEventDetails details) {
                        
            //Locate server side item object
            CallButtonItem item = callButtonItemMap.get(itemId);
            
            //Fire event
            fireEvent(new CallComponent.CallButtonItemClickEvent(CallComponent.this, item, details));
        }
        
        /* (non-Javadoc)
         * @see com.xmlnamespace.panel.vaadin.shared.ui.CallComponentServerRpc#updateTime()
         */
        @Override
        public void updateTime() {
            setStartTime(startTime);
        }
        
    };
    
    /**
     * Instantiates a new call component.
     */
    public CallComponent() {
        registerRpc(rpc);
    }
    
    /* (non-Javadoc)
     * @see com.vaadin.ui.AbstractComponent#attach()
     */
    @Override
    public void attach() {
        super.attach();
        
        // Update the conversation start time with the stored value, to handle page refreshes & detach/re-attach scenarios
        setStartTime(startTime);
    }

    /**
     * Sets the caller id.
     *
     * @param callerId the new caller id
     */
    public void setCallerId(String callerId) {
        getState().callerId = callerId;
        markAsDirty();
    }
    
    /**
     * Gets the caller id.
     *
     * @return the caller id
     */
    public String getCallerId() {
        return getState().callerId;
    }
    
    /**
     * Sets the extension.
     *
     * @param extension the new extension
     */
    public void setExtension(String extension) {
        getState().extension = extension;
        markAsDirty();
    }
    
    /**
     * Gets the extension.
     *
     * @return the extension
     */
    public String getExtension() {
        return getState().extension;
    }
   
    /**
     * Sets the start time in seconds. If the value is > 0 the
     * timer will be shown and will start counting at the given number
     * of seconds.
     * 
     * If -1 is given the timer will be hidden.
     *
     * @param seconds the number of seconds to start the timer at or -1 to hide the timer
     */
    public void setStartTime(long startTime) {
        this.startTime = startTime;
        if (startTime < 0)
            getRpcProxy(CallComponentClientRpc.class).setCurrentTime(startTime);
        else
            getRpcProxy(CallComponentClientRpc.class).setCurrentTime(new Date().getTime() - startTime);
    }
            
    /**
     * Gets the call drag object.
     *
     * @return the call drag object
     */
    public Object getCallDragObject() {
        return callDragObject;
    }

    /**
     * Sets the call drag object.
     * 
     * This is the object that will be dragged if a user starts a 
     * drag action on this call component.
     *
     * @param callDragObject the new call drag object or null for no call drag
     */
    public void setCallDragObject(Object callDragObject) {
        this.callDragObject = callDragObject;
        getState().hasDragObject = callDragObject != null;
        markAsDirty();
    }

    /**
     * Adds a call button item with the give icon.
     *
     * @param icon the icon that represents the button
     * @return the call button item created
     */
    public CallButtonItem addCallButtonItem(Resource icon) {

        // Determine the url reference for the button icon
        URLReference iconRef = null;
        if (icon != null) {
            iconRef = new ResourceReference(icon, this, "button-icon");
        }

        // Create item and add to the registry
        CallButtonItem item = new CallButtonItem(iconRef);
        getState().callButtonItems.add(item);
        callButtonItemMap.put(item.getId(), item);

        // Inform client of addition
        getRpcProxy(CallComponentClientRpc.class).addCallButtonItem(item);

        // Add listener to detect changes on the item
        item.addChangeListener(this);

        return item;
    }

    /**
     * Removes a call button item.
     *
     * @param item the item to remove
     */
    public void removeCallButtonItem(CallButtonItem item) {

        // Remove item change listener
        item.removeChangeListener(this);

        // Inform the client of the removal
        getRpcProxy(CallComponentClientRpc.class).removeCallButtonItem(item);

        // Remove item from the registry
        getState().callButtonItems.remove(item);
        callButtonItemMap.remove(item.getId());
    }

    /**
     * Convenience method used to set a new icon on a button item.
     * 
     * Users can also set a URLReference on the item itself
     *
     * @param item the item to set the icon on
     * @param icon the new icon
     */
    public void setButtonItemIcon(CallButtonItem item, Resource icon) {
        item.setIcon(new ResourceReference(icon, this, "button-icon"));
    }
    
    /**
     * Convenience method used to set a new tool tip on a button item.
     *
     * @param item the item to set the tool tip on
     * @param toolTip the tool tip to set
     */
    public void setButtonItemToolTip(CallButtonItem item, String toolTip) {
        item.setToolTip(toolTip);
    }

    /**
     * Sets the drag mode.
     * 
     * CallComponent.DragMode.NONE will disable all drags from the component
     * CallComponent.DragMode.DRAG will enable drags from the component
     *
     * @param dragMode the new drag mode
     */
    public void setDragMode(CallComponent.DragMode dragMode) {
        getState().dragMode = dragMode.ordinal();
        markAsDirty();
    }
    
    /**
     * Gets the drag mode.
     *
     * @return the drag mode
     */
    public CallComponent.DragMode getDragMode() {
        return CallComponent.DragMode.values()[getState().dragMode];
    }
        
    /**
     * Adds a listener to detect when a call button item is clicked
     *
     * @param listener the listener
     */
    public void addCallButtonItemClickListener(CallButtonItemClickListener listener) {
        addListener(CallButtonItemClickEvent.class, listener,
                CallButtonItemClickListener.clickMethod);
    }
    
    /**
     * Removes a call button item click listener.
     *
     * @param listener the listener
     */
    public void removeCallButtonItemClickListener(CallButtonItemClickListener listener) {
        removeListener(CallButtonItemClickEvent.class, listener,
                CallButtonItemClickListener.clickMethod);
    }
                    
    /* (non-Javadoc)
     * @see com.vaadin.ui.AbstractComponent#getState()
     */
    @Override
    protected CallComponentState getState() {
        return (CallComponentState) super.getState();
    }

    /* (non-Javadoc)
     * @see com.vaadin.event.dd.DragSource#getTransferable(java.util.Map)
     */
    @Override
    public Transferable getTransferable(Map<String, Object> rawVariables) {
        return new CallComponentTransferable(this, rawVariables);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.xmlnamespace.panel.vaadin.shared.ui.CallComponentItem.
     * CallComponentItemChangeListener
     * #callComponentItemChanged(com.xmlnamespace.
     * panel.vaadin.shared.ui.CallComponentItem.CallComponentItemChangedEvent)
     */
    @Override
    public void callComponentItemChanged(CallComponentItemChangedEvent event) {
        CallComponentItem item = event.getItem();
        getRpcProxy(CallComponentClientRpc.class).updateCallButtonItem((CallButtonItem) item);
    }
                
    /**
     * Base class for call item click events
     */
    public static class CallComponentItemClickEvent extends ClickEvent {

        /** The Constant serialVersionUID. */
        private static final long serialVersionUID = 1L;

        /** The item. */
        private CallComponentItem item;

        /**
         * Instantiates a new call component item click event.
         *
         * @param source the source
         * @param item the item
         * @param details the details
         */
        public CallComponentItemClickEvent(Component source, CallComponentItem item,
                MouseEventDetails details) {
            super(source, details);
            this.item = item;
        }

        /**
         * Gets the item that was clicked on.
         *
         * @return the item that was clicked on
         */
        public CallComponentItem getItem() {
            return item;
        }
        
    }
        
    /**
     * Event fired when a user clicks a button item
     */
    public static class CallButtonItemClickEvent extends CallComponentItemClickEvent {

        /** The Constant serialVersionUID. */
        private static final long serialVersionUID = 1L;

        /**
         * Instantiates a new call button item click event.
         *
         * @param source the source
         * @param item the item
         * @param details the details
         */
        public CallButtonItemClickEvent(Component source,
                CallButtonItem item, MouseEventDetails details) {
            super(source, item, details);
        }
        
        /* (non-Javadoc)
         * @see com.xmlnamespace.panel.vaadin.ui.CallComponent.CallComponentItemClickEvent#getItem()
         */
        @Override
        public CallButtonItem getItem() {
            return (CallButtonItem)super.getItem();
        }

    }
    
    /**
     * The listener interface for receiving callButtonItemClick events.
     * The class that is interested in processing a callButtonItemClick
     * event implements this interface, and the object created
     * with that class is registered with a component using the
     * component's <code>addCallButtonItemClickListener<code> method. When
     * the callButtonItemClick event occurs, that object's appropriate
     * method is invoked.
     *
     * @see CallButtonItemClickEvent
     */
    public interface CallButtonItemClickListener extends ClickListener {
    }
        
    /**
     * Drag modes for the call component.
     */
    public enum DragMode {
        
        /** Disables all drag functionality. */
        NONE,
        
        /** Enables drag functionality. */
        DRAG
    }
    
    /**
     * Transferable used for drags from the call component
     */
    public class CallComponentTransferable extends TransferableImpl {

        /** The object. */
        private Object object;
        
        /** The Constant serialVersionUID. */
        private static final long serialVersionUID = 1L;
        
        /** The Constant TYPE_ID. */
        private static final String TYPE_ID = "object-type";
        
        /** The Constant CALL_TYPE. */
        private static final String CALL_TYPE = "call";

        /**
         * Instantiates a new call component transferable.
         *
         * @param sourceComponent the source component
         * @param rawVariables the raw variables
         */
        public CallComponentTransferable(Component sourceComponent,
                Map<String, Object> rawVariables) {
            super(sourceComponent, rawVariables);
            
            String type = (String)getData(TYPE_ID);
            if(type != null) {
                if(type.equals(CALL_TYPE)) {
                    object = callDragObject;
                }
            }
        }
        
        /**
         * Gets the drag object.
         *
         * @return the drag object
         */
        public Object getObject() {
            return object;
        }
        
    }
        
}
