/*******************************************************************************
 * I9 TECHNOLOGIES CONFIDENTIAL
 * __________________
 * Copyright © 2013-2016 i9 Technologies 
 * All Rights Reserved.
 * 
 * NOTICE:  All information contained herein is, and remains the property 
 * of i9 Technologies and its suppliers, if any.  The intellectual and technical 
 * concepts contained herein are proprietary to i9 Technologies and its 
 * suppliers and may be covered by U.S. and Foreign Patents, patents in 
 * process, and are protected by trade secret or copyright law. Dissemination 
 * of this information or reproduction of this material is strictly forbidden 
 * unless prior written permission is obtained from i9 Technologies.
 ******************************************************************************/
package com.xmlnamespace.panel.vaadin.ui;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.vaadin.event.DataBoundTransferable;
import com.vaadin.event.MouseEvents.ClickEvent;
import com.vaadin.event.MouseEvents.ClickListener;
import com.vaadin.event.dd.DragAndDropEvent;
import com.vaadin.event.dd.DropHandler;
import com.vaadin.event.dd.DropTarget;
import com.vaadin.event.dd.TargetDetails;
import com.vaadin.event.dd.TargetDetailsImpl;
import com.vaadin.event.dd.acceptcriteria.AcceptCriterion;
import com.vaadin.event.dd.acceptcriteria.ServerSideCriterion;
import com.vaadin.server.PaintException;
import com.vaadin.server.PaintTarget;
import com.vaadin.server.Resource;
import com.vaadin.server.ResourceReference;
import com.vaadin.shared.MouseEventDetails;
import com.vaadin.ui.AbstractComponent;
import com.vaadin.ui.Component;
import com.vaadin.ui.LegacyComponent;
import com.vaadin.util.ReflectTools;
import com.xmlnamespace.panel.vaadin.shared.ui.DropDownMenuClientRpc;
import com.xmlnamespace.panel.vaadin.shared.ui.DropDownMenuItem;
import com.xmlnamespace.panel.vaadin.shared.ui.DropDownMenuItem.DropDownMenuItemChangeListener;
import com.xmlnamespace.panel.vaadin.shared.ui.DropDownMenuItem.DropDownMenuItemChangedEvent;
import com.xmlnamespace.panel.vaadin.shared.ui.DropDownMenuServerRpc;
import com.xmlnamespace.panel.vaadin.shared.ui.DropDownMenuState;
import com.xmlnamespace.panel.vaadin.ui.UserComponent.UserComponentTransferable;

/**
 * Component that acts as a drop down menu. When clicked or dragged over, a menu
 * overlay will appear below the component.
 * 
 * Menu items can be added and removed from the overlay menu via the following
 * methods:
 * 
 * <code>addMenuItem()</code> <code>removeMenuItem()</code>
 * 
 * Once added the methods on the items themselves can be called to update the
 * items.
 * 
 * Provides support for receiving click and drop events on menu items
 */
public class DropDownMenu extends AbstractComponent implements
        DropDownMenuItemChangeListener, DropTarget, LegacyComponent {

    private static final long serialVersionUID = 7114119874483614403L;

    /** Map of menu ids to menu items */
    private Map<Double, DropDownMenuItem> menuItemMap = new HashMap<Double, DropDownMenuItem>();

    /** The accepted drop classes. */
    private Class<?>[] acceptedDropClasses;

    /** The drop handler. */
    private DropDownMenuDropHandler dropHandler = new DropDownMenuDropHandler();

    /** The server side RPC . */
    private DropDownMenuServerRpc rpc = new DropDownMenuServerRpc() {

        private static final long serialVersionUID = 7867311537382989829L;

        @Override
        public void menuItemClicked(double itemId, MouseEventDetails details) {

            // Locate the server side item object
            DropDownMenuItem item = menuItemMap.get(itemId);

            // Fire event
            fireEvent(new MenuItemClickEvent(DropDownMenu.this, item, details));
        }
    };

    /**
     * Instantiates a new drop down menu.
     */
    public DropDownMenu() {
        registerRpc(rpc);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.vaadin.ui.AbstractComponent#getState()
     */
    @Override
    public DropDownMenuState getState() {
        return (DropDownMenuState) super.getState();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.vaadin.ui.AbstractComponent#setIcon(com.vaadin.server.Resource)
     */
    @Override
    public void setIcon(Resource icon) {
        if (icon == null) {
            getState().icon = null;
        } else {
            getState().icon = new ResourceReference(icon, this, "icon");
        }
        markAsDirty();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.vaadin.ui.AbstractComponent#setCaption(java.lang.String)
     */
    @Override
    public void setCaption(String caption) {
        getState().caption = caption != null ? caption : "";
        markAsDirty();
    }

    /**
     * Adds a menu item.
     *
     * @param name
     *            the name of the menu item
     * @param type
     *            the type of the menu item
     * @return the drop down menu item
     */
    public DropDownMenuItem addMenuItem(String name, String type) {

        // Create item and add to the registry
        DropDownMenuItem item = new DropDownMenuItem(name, type);
        getState().menuItems.add(item);
        menuItemMap.put(item.getId(), item);

        // Inform the client of addition
        getRpcProxy(DropDownMenuClientRpc.class).addMenuItem(item);

        // Add listener to detect changes on the item
        item.addChangeListener(this);

        return item;
    }

    /**
     * Removes a menu item.
     *
     * @param item
     *            the item to remove
     */
    public void removeMenuItem(DropDownMenuItem item) {

        // Remove item change listener
        item.removeChangeListener(this);

        // Inform the client of the removal
        getRpcProxy(DropDownMenuClientRpc.class).removeMenuItem(item);

        // Remove item from the registry
        getState().menuItems.remove(item);
        menuItemMap.remove(item.getId());
    }

    /**
     * Adds a menu item click listener.
     *
     * @param listener
     *            the listener to add
     */
    public void addMenuItemClickListener(MenuItemClickListener listener) {
        addListener(MenuItemClickEvent.class, listener,
                MenuItemClickListener.clickMethod);
    }

    /**
     * Removes a menu item click listener.
     *
     * @param listener
     *            the listener to remove
     */
    public void removeMenuItemClickListener(MenuItemClickListener listener) {
        removeListener(MenuItemClickEvent.class, listener,
                MenuItemClickListener.clickMethod);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.xmlnamespace.panel.vaadin.shared.DropDownMenuItem.
     * DropDownMenuItemChangeListener
     * #menuItemChanged(com.xmlnamespace.panel.vaadin
     * .shared.DropDownMenuItem.DropDownMenuItemChangedEvent)
     */
    @Override
    public void menuItemChanged(DropDownMenuItemChangedEvent event) {

        DropDownMenuItem item = event.getItem();

        // Inform client of item update
        if (item instanceof DropDownMenuItem) {
            getRpcProxy(DropDownMenuClientRpc.class).updateMenuItem(item);
        }
    }

    /**
     * Removes all menu items.
     */
    public void removeAllMenuItems() {

        // We need to make a copy of the set to iterate over. If we use
        // the same map we will get a ConcurrentModification exception
        // as we are removing items.
        Set<DropDownMenuItem> items = new HashSet<DropDownMenuItem>(
                menuItemMap.values());
        for (DropDownMenuItem item : items) {
            removeMenuItem(item);
        }
        getState().menuItems = new ArrayList<DropDownMenuItem>();
    }

    /**
     * Gets the accepted drop classes.
     *
     * @return the accepted drop classes
     */
    public Class<?>[] getAcceptedDropClasses() {
        return acceptedDropClasses;
    }

    /**
     * Sets the accepted drop classes.
     *
     * @param acceptedDropClasses
     *            the new accepted drop classes
     */
    public void setAcceptedDropClasses(Class<?>[] acceptedDropClasses) {
        this.acceptedDropClasses = acceptedDropClasses;
        markAsDirty();
    }

    /**
     * Adds an object drop listener.
     *
     * @param listener
     *            the listener to add
     */
    public void addObjectDropListener(ObjectDropListener listener) {
        addListener(ObjectDropEvent.class, listener,
                ObjectDropListener.DROP_METHOD);
    }

    /**
     * Removes an object drop listener.
     *
     * @param listener
     *            the listener to remove
     */
    public void removeObjectDropListener(ObjectDropListener listener) {
        removeListener(ObjectDropEvent.class, listener,
                ObjectDropListener.DROP_METHOD);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.vaadin.event.dd.DropTarget#getDropHandler()
     */
    @Override
    public DropHandler getDropHandler() {
        return dropHandler;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * com.vaadin.event.dd.DropTarget#translateDropTargetDetails(java.util.Map)
     */
    @Override
    public TargetDetails translateDropTargetDetails(
            Map<String, Object> clientVariables) {
        return new TargetDetailsImpl(clientVariables, this);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * com.vaadin.ui.LegacyComponent#paintContent(com.vaadin.server.PaintTarget)
     */
    @Override
    public void paintContent(PaintTarget target) throws PaintException {

        // Only add the drop handler if there are accepted drop classes
        if (acceptedDropClasses != null && acceptedDropClasses.length != 0) {
            dropHandler.getAcceptCriterion().paint(target);
        }
    }

    /**
     * Gets the data from drag and drop event.
     *
     * @param event
     *            the event
     * @return the data from drag and drop event
     */
    private static Object getDataFromDragAndDropEvent(DragAndDropEvent event) {

        // If drop is a DragAndDropComponent grab it from the custom method else
        // if it is a DataBoundTransferable use the get item id class
        if (event.getTransferable().getSourceComponent() instanceof DragAndDropComponent) {
            return ((DragAndDropComponent) event.getTransferable()
                    .getSourceComponent()).getDragObject();
        } else if (event.getTransferable() instanceof DataBoundTransferable) {
            return ((DataBoundTransferable) event.getTransferable())
                    .getItemId();
        } else if (event.getTransferable() instanceof UserComponentTransferable) {
            return ((UserComponentTransferable) event.getTransferable())
                    .getObject();
        } else if (event.getTransferable() instanceof CallComponent.CallComponentTransferable) {
            return ((CallComponent.CallComponentTransferable) event
                    .getTransferable()).getObject();
        } else {
            throw new IllegalArgumentException("Drop type not known");
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.vaadin.server.VariableOwner#changeVariables(java.lang.Object,
     * java.util.Map)
     */
    @Override
    public void changeVariables(Object source, Map<String, Object> variables) {
    }

    /**
     * The listener interface for receiving menuItemClick events. The class that
     * is interested in processing a menuItemClick event implements this
     * interface, and the object created with that class is registered with a
     * component using the component's
     * <code>addMenuItemClickListener<code> method. When
     * the menuItemClick event occurs, that object's appropriate
     * method is invoked.
     *
     * @see MenuItemClickEvent
     */
    public interface MenuItemClickListener extends ClickListener {
    }

    /**
     * Event fired when a menu item is clicked
     */
    public static class MenuItemClickEvent extends ClickEvent {

        private static final long serialVersionUID = -6508220945423449098L;

        /** The item that was clicked. */
        private DropDownMenuItem item;

        /**
         * Instantiates a new menu item click event.
         *
         * @param source
         *            the source component of the event
         * @param item
         *            the menu item that was clicked
         * @param details
         *            the mouse event details
         */
        public MenuItemClickEvent(Component source, DropDownMenuItem item,
                MouseEventDetails details) {
            super(source, details);
            this.item = item;
        }

        /**
         * Gets the item that was clicked.
         *
         * @return the item that was clicked
         */
        public DropDownMenuItem getItem() {
            return item;
        }

    }

    /**
     * Event fired when an object is dropped on a menu item
     */
    public static class ObjectDropEvent extends Component.Event {

        private static final long serialVersionUID = 8619722791145177383L;

        /** The drag and drop event. */
        private DragAndDropEvent event;

        /** The object that was dropped. */
        private Object dropObject;

        /** The menu item that was dropped on. */
        private DropDownMenuItem dropItem;

        /**
         * Instantiates a new object drop event.
         *
         * @param event
         *            the drag and drop event
         * @param source
         *            component of the event
         * @param dropObject
         *            the object that was dropped
         * @param dropItem
         *            the menu item that was dropped on
         */
        public ObjectDropEvent(DragAndDropEvent event, Component source,
                Object dropObject, DropDownMenuItem dropItem) {
            super(source);
            this.event = event;
            this.dropObject = dropObject;
            this.dropItem = dropItem;
        }

        /**
         * Gets the object that was dropped.
         *
         * @return the object that was dropped
         */
        public Object getDropObject() {
            return dropObject;
        }

        /**
         * Gets the menu item that was dropped on.
         *
         * @return the menu item that was dropped on or null if the drop did not
         *         occur on a menu item
         */
        public DropDownMenuItem getDropItem() {
            return dropItem;
        }

        /**
         * Gets the DragAndDropEvent that caused this event.
         *
         * @return the drag and drop event
         */
        public DragAndDropEvent getEvent() {
            return event;
        }

    }

    /**
     * The listener interface for receiving objectDrop events. The class that is
     * interested in processing a objectDrop event implements this interface,
     * and the object created with that class is registered with a component
     * using the component's <code>addObjectDropListener<code> method. When
     * the objectDrop event occurs, that object's appropriate
     * method is invoked.
     *
     * @see ObjectDropEvent
     */
    public interface ObjectDropListener extends Serializable {

        /** The drop method. */
        public static final Method DROP_METHOD = ReflectTools.findMethod(
                ObjectDropListener.class, "onDrop", ObjectDropEvent.class);

        /**
         * Called when a drop occurs on this component
         *
         * @param event
         *            the object drop event
         */
        public void onDrop(ObjectDropEvent event);

    }

    /**
     * Drop handler for the component
     */
    private class DropDownMenuDropHandler implements DropHandler {

        private static final long serialVersionUID = -805148233638690619L;

        /** Property name for the item id that was dropped on. */
        private static final String DROP_ITEM_DETAIL_ID = "item-id-dropped-on";

        /*
         * (non-Javadoc)
         * 
         * @see
         * com.vaadin.event.dd.DropHandler#drop(com.vaadin.event.dd.DragAndDropEvent
         * )
         */
        @Override
        public void drop(DragAndDropEvent event) {

            // Grab the dropped object
            Object dropObject = getDataFromDragAndDropEvent(event);

            // Determine if a user item was dropped on
            Double itemId = (Double) event.getTargetDetails().getData(
                    DROP_ITEM_DETAIL_ID);

            // Get the item
            DropDownMenuItem item = null;
            if (itemId != null) {
                item = menuItemMap.get(itemId);
            }

            // Fire drop event
            fireEvent(new DropDownMenu.ObjectDropEvent(event,
                    DropDownMenu.this, dropObject, item));
        }

        /*
         * (non-Javadoc)
         * 
         * @see com.vaadin.event.dd.DropHandler#getAcceptCriterion()
         */
        @Override
        public AcceptCriterion getAcceptCriterion() {
            return new ClassTypeCriterion();
        }

    }

    /**
     * Server side accept criterion that accepts specific classes
     */
    public class ClassTypeCriterion extends ServerSideCriterion {

        private static final long serialVersionUID = 8789180045404461343L;

        /*
         * (non-Javadoc)
         * 
         * @see
         * com.vaadin.event.dd.acceptcriteria.AcceptCriterion#accept(com.vaadin
         * .event.dd.DragAndDropEvent)
         */
        @Override
        public boolean accept(DragAndDropEvent dragEvent) {

            // Check if the destination is the same as the source if so don't
            // accept
            if (dragEvent.getTransferable().getSourceComponent()
                    .equals(DropDownMenu.this)) {
                return false;
            }

            // If accept classes are set check if any match if so accept
            if (acceptedDropClasses != null) {
                Object data = null;
                try {
                    data = getDataFromDragAndDropEvent(dragEvent);
                } catch (Exception e) {
                    return false;
                }

                for (Class<?> classType : acceptedDropClasses) {
                    if (classType.isAssignableFrom(data.getClass())) {
                        return true;
                    }
                }
            }

            return false;
        }
    }

}
